@@ -4,7 +4,8 @@ use strict;
 use warnings;
 use Carp;
 
-use Dancer::Object;
+use base 'Dancer::Object';
+
 use Dancer::Request::Upload;
 use Dancer::SharedData;
 use Encode;
@@ -12,7 +13,6 @@ use HTTP::Body;
 use URI;
 use URI::Escape;
 
-use base 'Dancer::Object';
 my @http_env_keys = (
     'user_agent',      'host',       'accept_language', 'accept_charset',
     'accept_encoding', 'keep_alive', 'connection',      'accept',
@@ -20,12 +20,12 @@ my @http_env_keys = (
 );
 my $count = 0;
 
-Dancer::Request->attributes(
+__PACKAGE__->attributes(
 
     # query
     'env',          'path',    'method',
     'content_type', 'content_length',
-    'body',         'id',      'request_uri',
+    'body',         'id',
     'uploads',      'headers', 'path_info',
     'ajax', 
     @http_env_keys,
@@ -33,8 +33,19 @@ Dancer::Request->attributes(
 
 # aliases
 sub agent                 { $_[0]->user_agent }
-sub remote_address        { $_[0]->{env}->{'REMOTE_ADDR'} }
-sub forwarded_for_address { $_[0]->{env}->{'X_FORWARDED_FOR'} }
+sub remote_address        { $_[0]->address }
+sub forwarded_for_address { $_[0]->env->{'X_FORWARDED_FOR'} }
+sub address               { $_[0]->env->{REMOTE_ADDR} }
+sub remote_host           { $_[0]->env->{REMOTE_HOST} }
+sub protocol              { $_[0]->env->{SERVER_PROTOCOL} }
+sub port                  { $_[0]->env->{SERVER_PORT} }
+sub request_uri           { $_[0]->env->{REQUEST_URI} }
+sub user                  { $_[0]->env->{REMOTE_USER} }
+sub script_name           { $_[0]->env->{SCRIPT_NAME} }
+sub scheme                { $_[0]->env->{'psgi.url_scheme'} }
+sub secure                { $_[0]->scheme eq 'https' }
+sub uri                   { $_[0]->request_uri }
+
 sub is_head               { $_[0]->{method} eq 'HEAD' }
 sub is_post               { $_[0]->{method} eq 'POST' }
 sub is_get                { $_[0]->{method} eq 'GET' }
@@ -45,7 +56,7 @@ sub header                { $_[0]->{headers}->header($_[1]) }
 # public interface compat with CGI.pm objects
 sub request_method { method(@_) }
 sub Vars           { params(@_) }
-sub input_handle   { $_[0]->{env}->{'psgi.input'} || $_[0]->{env}->{'PSGI.INPUT'} }
+sub input_handle   { $_[0]->env->{'psgi.input'} || $_[0]->env->{'PSGI.INPUT'} }
 
 sub new {
     my ($class, $env) = @_;
@@ -95,14 +106,38 @@ sub new_for_request {
     return $req;
 }
 
+#Create a new request which is a clone of the current one, apart
+#from the path location, which points instead to the new location
+sub forward {
+    my ($class, $request, $to) = @_;
+
+    my $env = $request->env;
+    $env->{PATH_INFO} = $to;
+
+    my $new_request = $class->new($env);
+
+    $new_request->{params}  = $request->params;
+    $new_request->{body}    = $request->body;
+    $new_request->{headers} = $request->headers;
+
+    return $new_request;
+}
+
 sub base {
     my $self = shift;
+    my $uri  = $self->_common_uri;
+
+    return $uri->canonical;
+}
+
+sub _common_uri {
+    my $self = shift;
 
     my @env_names = qw(
       SERVER_NAME HTTP_HOST SERVER_PORT SCRIPT_NAME psgi.url_scheme
     );
 
-    my ($server, $host, $port, $path, $scheme) = @{$self->{env}}{@env_names};
+    my ($server, $host, $port, $path, $scheme) = @{$self->env}{@env_names};
 
     $scheme ||= $self->{'env'}{'PSGI.URL_SCHEME'};    # Windows
 
@@ -111,7 +146,19 @@ sub base {
     $uri->authority($host || "$server:$port");
     $uri->path($path      || '/');
 
-    return $uri->canonical;
+    return $uri;
+}
+
+sub uri_base {
+    my $self  = shift;
+    my $uri   = $self->_common_uri;
+    my $canon = $uri->canonical;
+
+    if ( $uri->path eq '/' ) {
+        $canon =~ s{/$}{};
+    }
+
+    return $canon;
 }
 
 sub uri_for {
@@ -129,7 +176,6 @@ sub uri_for {
     return $dont_escape ? uri_unescape($uri->canonical) : $uri->canonical;
 }
 
-
 sub params {
     my ($self, $source) = @_;
 
@@ -212,8 +258,8 @@ sub _init {
     $self->_build_headers();
     $self->_build_request_env();
     $self->_build_path()      unless $self->path;
-    $self->_build_method()    unless $self->method;
     $self->_build_path_info() unless $self->path_info;
+    $self->_build_method()    unless $self->method;
 
     $self->{_http_body} =
       HTTP::Body->new($self->content_type, $self->content_length);
@@ -250,17 +296,17 @@ sub _build_request_env {
    # Don't refactor that, it's called whenever a request object is needed, that
    # means at least once per request. If refactored in a loop, this will cost 4
    # times more than the following static map.
-    $self->{user_agent}       = $self->{env}{HTTP_USER_AGENT};
-    $self->{host}             = $self->{env}{HTTP_HOST};
-    $self->{accept_language}  = $self->{env}{HTTP_ACCEPT_LANGUAGE};
-    $self->{accept_charset}   = $self->{env}{HTTP_ACCEPT_CHARSET};
-    $self->{accept_encoding}  = $self->{env}{HTTP_ACCEPT_ENCODING};
-    $self->{keep_alive}       = $self->{env}{HTTP_KEEP_ALIVE};
-    $self->{connection}       = $self->{env}{HTTP_CONNECTION};
-    $self->{accept}           = $self->{env}{HTTP_ACCEPT};
-    $self->{accept_type}      = $self->{env}{HTTP_ACCEPT_TYPE};
-    $self->{referer}          = $self->{env}{HTTP_REFERER};
-    $self->{x_requested_with} = $self->{env}{HTTP_X_REQUESTED_WITH};
+    $self->{user_agent}       = $self->env->{HTTP_USER_AGENT};
+    $self->{host}             = $self->env->{HTTP_HOST};
+    $self->{accept_language}  = $self->env->{HTTP_ACCEPT_LANGUAGE};
+    $self->{accept_charset}   = $self->env->{HTTP_ACCEPT_CHARSET};
+    $self->{accept_encoding}  = $self->env->{HTTP_ACCEPT_ENCODING};
+    $self->{keep_alive}       = $self->env->{HTTP_KEEP_ALIVE};
+    $self->{connection}       = $self->env->{HTTP_CONNECTION};
+    $self->{accept}           = $self->env->{HTTP_ACCEPT};
+    $self->{accept_type}      = $self->env->{HTTP_ACCEPT_TYPE};
+    $self->{referer}          = $self->env->{HTTP_REFERER};
+    $self->{x_requested_with} = $self->env->{HTTP_X_REQUESTED_WITH};
 }
 
 sub _build_headers {
@@ -294,16 +340,13 @@ sub _build_path {
     my ($self) = @_;
     my $path = "";
 
-    $path .= $self->{env}{'SCRIPT_NAME'}
-      if defined $self->{env}->{'SCRIPT_NAME'};
-    $path .= $self->{env}->{'PATH_INFO'}
-      if defined $self->{env}->{'PATH_INFO'};
+    $path .= $self->script_name if defined $self->script_name;
+    $path .= $self->env->{PATH_INFO} if defined $self->env->{PATH_INFO};
 
     # fallback to REQUEST_URI if nothing found
     # we have to decode it, according to PSGI specs.
-    if (defined $self->{env}->{REQUEST_URI}) {
-        $self->{request_uri} = $self->{env}->{REQUEST_URI};
-        $path ||= $self->_url_decode($self->{request_uri});
+    if (defined $self->request_uri) {
+        $path ||= $self->_url_decode($self->request_uri);
     }
 
     croak "Cannot resolve path" if not $path;
@@ -312,7 +355,7 @@ sub _build_path {
 
 sub _build_path_info {
     my ($self) = @_;
-    my $info = $self->{env}->{'PATH_INFO'};
+    my $info = $self->env->{PATH_INFO};
     if (defined $info) {
 
         # Empty path info will be interpreted as "root".
@@ -326,7 +369,7 @@ sub _build_path_info {
 
 sub _build_method {
     my ($self) = @_;
-    $self->{method} = $self->{env}->{REQUEST_METHOD}
+    $self->{method} = $self->env->{REQUEST_METHOD}
       || $self->{request}->request_method();
 }
 
@@ -351,7 +394,7 @@ sub _parse_get_params {
     return $self->{_query_params} if defined $self->{_query_params};
     $self->{_query_params} = {};
 
-    my $source = $self->{env}{QUERY_STRING} || '';
+    my $source = $self->env->{QUERY_STRING} || '';
     foreach my $token (split /[&;]/, $source) {
         my ($key, $val) = split(/=/, $token);
         next unless defined $key;
@@ -402,7 +445,7 @@ sub _has_something_to_read {
 # taken from Miyagawa's Plack::Request::BodyParser
 sub _read {
     my ($self,)   = @_;
-    my $remaining = $self->{env}->{CONTENT_LENGTH} - $self->{_read_position};
+    my $remaining = $self->content_length - $self->{_read_position};
     my $maxlength = $self->{_chunk_size};
 
     return if ($remaining <= 0);
@@ -426,7 +469,7 @@ sub _read {
 sub _build_uploads {
     my ($self) = @_;
 
-    my $uploads = $self->{_http_body}->upload;
+    my $uploads = _decode($self->{_http_body}->upload);
     my %uploads;
 
     for my $name (keys %{$uploads}) {
@@ -497,6 +540,12 @@ objects. It uses the environment hash table given to build the request object.
 An alternate constructor convinient for test scripts which creates a request
 object with the arguments given.
 
+=head2 forward($request, $new_location)
+
+Create a new request which is a clone of the current one, apart
+from the path location, which points instead to the new location.
+This is used internally to chain requests using the forward keyword.
+
 =head2 to_string()
 
 Return a string represeting the request object (eg: C<"GET /some/path">)
@@ -509,6 +558,46 @@ While this method returns the method string as provided by the environment, it's
 better to use one of the following boolean accessors if you want to inspect the
 requested method.
 
+=head2 address()
+
+Return the IP address of the client.
+
+=head2 remote_host()
+
+Return the remote host of the client.
+
+=head2 protocol()
+
+Return the protocol (HTTP/1.0 or HTTP/1.1) used for the request.
+
+=head2 port()
+
+Return the port of the server.
+
+=head2 uri()
+
+An alias to request_uri()
+
+=head2 request_uri()
+
+Return the raw, undecoded request URI path.
+
+=head2 user()
+
+Return remote user if defined.
+
+=head2 script_name()
+
+Return script_name from the environment.
+
+=head2 scheme()
+
+Return the scheme of the request
+
+=head2 secure()
+
+Return true of false, indicating wether the connection is secure
+
 =head2 is_get()
 
 Return true if the method requested by the client is 'GET'
@@ -535,14 +624,27 @@ Return the path requested by the client.
 
 =head2 base()
 
-Returns an absolute URI for the base of the application
+Returns an absolute URI for the base of the application.  Returns a L<URI>
+object (which stringifies to the URL, as you'd expect).
+
+=head2 uri_base()
+
+Same thing as C<base> above, except it removes the last trailing slash in the
+path if it is the only path.
+
+This means that if your base is I<http://myserver/>, C<uri_base> will return
+I<http://myserver> (notice no trailing slash). This is considered very useful
+when using templates to do the following thing:
+
+    <link rel="stylesheet" href="<% request.uri_base %>/css/style.css" />
 
 =head2 uri_for(path, params)
 
 Constructs a URI from the base and the passed path.  If params (hashref) is
 supplied, these are added to the query string of the uri.  If the base is
 C<http://localhost:5000/foo>, C<< request->uri_for('/bar', { baz => 'baz' }) >>
-would return C<http://localhost:5000/foo/bar?baz=baz>.
+would return C<http://localhost:5000/foo/bar?baz=baz>.  Returns a L<URI> object
+(which stringifies to the URL, as you'd expect).
 
 =head2 params($source)
 
@@ -662,30 +764,37 @@ Dancer::Request object through specific accessors, here are those supported:
 
 =over 4
 
-=item C<user_agent>
-
-=item C<agent> (alias for C<user_agent>)
-
-=item C<host>
-
-=item C<accept_language>
+=item C<accept>
 
 =item C<accept_charset>
 
 =item C<accept_encoding>
 
-=item C<keep_alive>
+=item C<accept_language>
+
+=item C<accept_type>
+
+=item C<agent> (alias for C<user_agent>)
 
 =item C<connection>
 
-=item C<accept>
-
 =item C<forwarded_for_address>
 
+=item C<host>
+
+=item C<keep_alive>
+
+=item C<path_info>
+
+=item C<referer>
+
 =item C<remote_address>
 
+=item C<user_agent>
+
 =back
 
+
 =head1 AUTHORS
 
 This module has been written by Alexis Sukrieh and was mostly
