@@ -48,10 +48,9 @@ sub destroy {
 # This is the default constructor for the session object, the only mandatory
 # attribute is 'id'. The whole object should be serialized by the session
 # engine.
-sub new {
-    my $self = Dancer::Object::new(@_);
+sub init {
+    my ($self) = @_;
     $self->id(build_id());
-    return $self;
 }
 
 # session name can be set in configuration file:
@@ -63,6 +62,7 @@ sub session_name {
     setting('session_name') || 'dancer.session';
 }
 
+
 # we try to make the best random number
 # with native Perl 5 code.
 # to rebuild a session id, an attacker should know:
@@ -90,15 +90,17 @@ sub read_session_id {
 sub write_session_id {
     my ($class, $id) = @_;
     my %cookie = (
-        name  => $SESSION_NAME,
-        value => $id
+        name   => $SESSION_NAME,
+        value  => $id,
+        secure => setting('session_secure')
     );
     if (my $expires = setting('session_expires')) {
         $cookie{expires} =
           Dancer::Cookie::_epoch_to_gmtstring(time + $expires);
     }
 
-    Dancer::Cookies->cookies->{$SESSION_NAME} = Dancer::Cookie->new(%cookie);
+    my $c = Dancer::Cookie->new(%cookie);
+    Dancer::Cookies->set_cookie_object($SESSION_NAME => $c);
 }
 
 1;
@@ -149,10 +151,21 @@ done in order to allow multiple session storage with a common interface.
 Any session engine must inherits from Dancer::Session::Abstract and implement
 the following abstract methods.
 
+=head2 Configuration
+
+These settings effect how a session acts.
+
+=head3 session_name
+
 The default session name is "dancer_session". This can be set in your config file:
 
     setting session_name: "mydancer_session"
 
+=head3 session_secure
+
+The user's session id is stored in a cookie.  If true, this cookie
+will be made "secure" meaning it will only be served over https.
+
 =head2 Abstract Methods
 
 =over 4
