@@ -3,7 +3,10 @@ package Dancer::Template::Abstract;
 use strict;
 use warnings;
 use Carp;
+
+use Dancer::Deprecation;
 use Dancer::FileUtils 'path';
+
 use base 'Dancer::Engine';
 
 # Overloads this method to implement the rendering
@@ -16,7 +19,8 @@ sub default_tmpl_ext { "tt" }
 sub _template_name {
     my ( $self, $view ) = @_;
     my $def_tmpl_ext = $self->config->{extension} || $self->default_tmpl_ext();
-    $view .= ".$def_tmpl_ext" if $view !~ /\.${def_tmpl_ext}$/;
+    $view .= ".$def_tmpl_ext" if $view !~ /\.\Q$def_tmpl_ext\E$/;
+    return $view;
 }
 
 sub view {
@@ -45,7 +49,6 @@ sub apply_renderer {
     ($tokens, undef) = _prepare_tokens_options($tokens);
 
     $view = $self->view($view);
-    -r $view or return;
 
     $_->($tokens) for (@{Dancer::App->current->registry->hooks->{before_template}});
 
@@ -90,6 +93,7 @@ sub _prepare_tokens_options {
 
     # these are the default tokens provided for template processing
     $tokens ||= {};
+    $tokens->{perl_version}   = $];
     $tokens->{dancer_version} = $Dancer::VERSION;
     $tokens->{settings}       = Dancer::Config->settings;
     $tokens->{request}        = Dancer::SharedData->request;
@@ -101,6 +105,46 @@ sub _prepare_tokens_options {
     return ($tokens, $options);
 }
 
+sub _render_with_layout {
+    my ($class, $content, $tokens, $options) = @_;
+
+    Dancer::Deprecation::deprecated(
+        feature => 'render_with_layout',
+        version => '1.3000',
+        reason  => "use the 'engine' keyword to get the template engine, and use 'apply_layout' on the result",
+    );
+
+    my $full_content = Dancer::Template->engine->apply_layout($content, $tokens, $options);
+
+    if (! defined $full_content) {
+          return Dancer::Error->new(
+            code    => 404,
+            message => "Page not found",
+        )->render();
+    }
+    return $full_content;
+}
+
+sub template {
+    my ($class, $view, $tokens, $options) = @_;
+    my ($content, $full_content);
+
+    $options ||= {};
+    $content = $view ? Dancer::Template->engine->apply_renderer($view, $tokens)
+                     : delete $options->{content};
+
+    defined $content and $full_content = 
+      Dancer::Template->engine->apply_layout($content, $tokens, $options);
+
+    defined $full_content
+      and return $full_content;
+
+    Dancer::Error->new(
+        code    => 404,
+        message => "Page not found",
+    )->render();
+}
+
 1;
 __END__
 
@@ -129,13 +173,14 @@ The base class provides a plain init() method that only returns true.
 =item B<default_tmpl_ext()>
 
 Template class that inherits this class should override this method to return a default template
-extension, example: for Template::Toolkit it returns "tt" and for HTML::Mason it returns "mason". 
+extension, example: for Template::Toolkit it returns "tt" and for HTML::Mason it returns "mason".
 So when you call C<template 'index';> in your dispatch code, Dancer will look for a file 'index.tt'
 or 'index.mason' based on the template you use.
 
-Note 1: when returning the extension string, please do not add a dot in front of the extension 
-as Dancer will do that. 
-Note 2: for backwords compatibility abstract class returns "tt" instead of throwing
+Note 1: when returning the extension string, please do not add a dot in front of the extension
+as Dancer will do that.
+
+Note 2: for backwards compatibility abstract class returns "tt" instead of throwing
 an exception 'method not implemented'.
 
 User would be able to change the default extension using the
